import pytest

from sagemaker_tidymodels import Tidymodels, TidymodelsModel, get_role
from sagemaker.predictor import json_deserializer

s3_training_data = (
    "s3://sagemaker-sample-data-us-east-2/processing/census/census-income.csv"
)


@pytest.mark.parametrize("entry_point", ["tests/train.R", "tests/train-adv.R"])
def test_local_trainining(entry_point):
    tidymodels = Tidymodels(
        entry_point=entry_point,
        train_instance_type="local",
        role=get_role(),
        image_name="sagemaker-tidymodels",
    )

    tidymodels.fit({"train": s3_training_data})


with open("tests/example-data.csv") as example_data:
    example_data = example_data.read()


def test_local_endpoint():
    model_data = "s3://sagemaker-us-east-2-495577990003/sagemaker-tidymodels-2020-07-26-02-38-10-117/model.tar.gz"
    source_dir = "s3://sagemaker-us-east-2-495577990003/sagemaker-tidymodels-2020-07-26-23-37-01-218/source/sourcedir.tar.gz"

    model = TidymodelsModel(
        model_data=model_data,
        role=get_role(),
        entry_point="tests/train-adv.R",
        source_dir=source_dir,
        image="sagemaker-tidymodels",
    )

    predictor = model.deploy(initial_instance_count=1, instance_type="local")
    predictor.content_type = "application/json"
    predictor.deserializer = json_deserializer

    predicted_value = predictor.predict(example_data)

    predictor.delete_endpoint()

    assert predicted_value == ["- 50000.\n"]


## TODO:
##   add test with custom serve functions



from sagemaker.estimator import Framework
from sagemaker.model import FrameworkModel
from sagemaker.predictor import RealTimePredictor
import subprocess


def get_role(profile="sagemaker"):
    command = "aws configure get role_arn --profile {}".format(profile)
    return (
        subprocess.run(command.split(" "), stdout=subprocess.PIPE,)
        .stdout.decode("utf-8")
        .strip()
    )


class TidymodelsPredictor(RealTimePredictor):
    def __init__(self, endpoint_name, sagemaker_session=None, **kwargs):
        super(TidymodelsPredictor, self).__init__(
            endpoint_name, sagemaker_session=sagemaker_session, **kwargs
        )


class TidymodelsModel(FrameworkModel):

    # `FrameworkModel` accepts a `dependencies` argument to make more code availabe
    # in `/opt/ml/code`: https://github.com/aws/sagemaker-python-sdk/blob/8b2d5c8d73236b59bca6fdcaf96f227a01488288/src/sagemaker/model.py#L704-L712

    __framework_name__ = "tidymodels"

    def __init__(
        self,
        model_data,
        image,
        role,
        entry_point,
        predictor_cls=TidymodelsPredictor,
        **kwargs
    ):
        super(TidymodelsModel, self).__init__(
            model_data, image, role, entry_point, predictor_cls=predictor_cls, **kwargs
        )


class Tidymodels(Framework):
    def __init__(self, entry_point, image_name, role, train_instance_type, **kwargs):
        train_instance_count = kwargs.get("train_instance_count")
        if train_instance_count:
            if train_instance_count != 1:
                raise AttributeError(
                    "Tidymodels does not support distributed training. "
                    "Please remove the 'train_instance_count' argument or set "
                    "'train_instance_count=1' when initializing SKLearn."
                )

        super(Tidymodels, self).__init__(
            entry_point=entry_point,
            image_name=image_name,
            role=role,
            train_instance_type=train_instance_type,
            **dict(kwargs, train_instance_count=1)
        )

    def create_model(self, **kwargs):
        return TidymodelsModel(
            model_data=self.model_data,
            image=self.image_name,
            role=self.role,
            entry_point=self.entry_point,
            **kwargs
        )



from sagemaker_tidymodels import Tidymodels, get_role

tidymodels = Tidymodels(
    entry_point="tests/train.R",
    train_instance_type="local",
    role=get_role(),
    image_name="tmastny/sagemaker-tidymodels:latest",
)

s3_data = "s3://sagemaker-sample-data-us-east-2/processing/census/census-income.csv"
tidymodels.fit({"train": s3_data})



#!/usr/bin/env python

import os
import sys
import setuptools

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

readme = open("README.md").read()
requirements = open("requirements.txt").read()

setup(
    name="sagemaker-tidymodels",
    version="0.1.0",
    description="Sagemaker framework for Tidymodels",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Timothy Mastny",
    author_email="tim.mastny@gmail.com",
    url="https://github.com/tmastny/sagemaker-tidymodels",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
)
